@@ -1,13 +1,18 @@
 # Configuration file for the Sphinx documentation builder.
 #
+import os
+import time
 import datetime
 
-year = datetime.datetime.now().year
+
+year = datetime.datetime.utcfromtimestamp(
+    int(os.environ.get('SOURCE_DATE_EPOCH', time.time()))
+).year
 
 project = 'bitstring'
 copyright = f'2006 - {year}, Scott Griffiths'
 author = 'Scott Griffiths'
-release = '4.0'
+release = '4.1'
 
 extensions = []
 
@@ -26,9 +31,11 @@ html_css_files = ["custom.css"]
 html_theme = 'piccolo_theme'
 
 html_theme_options = {
-    "banner_text": "New major version released. Requires Python 3.7 or later - see release notes for full details.",
-    "banner_hiding": "permanent",
+    # "banner_text": "New major version released. Requires Python 3.7 or later - see release notes for full details.",
+    # "banner_hiding": "permanent",
     "show_theme_credit": False,
     "globaltoc_maxdepth": 2,
     "source_url": 'https://github.com/scott-griffiths/bitstring/',
 }
+
+html_logo = './bitstring_logo_small_white.png'
