@@ -2,14 +2,13 @@
 import io
 import unittest
 import sys
-
-sys.path.insert(0, '..')
+import bitarray
 import bitstring
 import array
 import os
-from bitstring import InterpretError
-from bitstring import MmapByteArray
-from bitstring import Bits, BitArray, ByteStore
+from bitstring import InterpretError, Bits, BitArray
+
+sys.path.insert(0, '..')
 
 THIS_DIR = os.path.dirname(os.path.abspath(__file__))
 
@@ -93,6 +92,7 @@ class Creation(unittest.TestCase):
 
     def testCreationFromInt(self):
         s = Bits(int=0, length=4)
+        temp = s.hex
         self.assertEqual(s.bin, '0000')
         s = Bits(int=1, length=2)
         self.assertEqual(s.bin, '01')
@@ -172,9 +172,40 @@ class Creation(unittest.TestCase):
         with self.assertRaises(bitstring.CreationError):
             Bits(squirrel=5)
 
-    def testDataStoreType(self):
-        a = Bits('0xf')
-        self.assertEqual(type(a._datastore), bitstring.ByteStore)
+    def testCreationFromBitarray(self):
+        ba = bitarray.bitarray('0010')
+        bs = Bits(ba)
+        self.assertEqual(bs.bin, '0010')
+        bs = Bits(ba, length=2)
+        self.assertEqual(bs.bin, '00')
+        bs2 = Bits(bitarray=ba)
+        self.assertEqual(bs2.bin, '0010')
+
+    def testCreationFromFrozenBitarray(self):
+        fba = bitarray.frozenbitarray('111100001')
+        ba = Bits(fba)
+        self.assertEqual(ba.bin, '111100001')
+        bs2 = Bits(bitarray=fba)
+        self.assertEqual(bs2.bin, '111100001')
+        bs3 = Bits(bitarray=fba, offset=4)
+        self.assertEqual(bs3.bin, '00001')
+        bs3 = Bits(bitarray=fba, offset=4, length=4)
+        self.assertEqual(bs3.bin, '0000')
+
+    def testCreationFromBitarrayErrors(self):
+        ba = bitarray.bitarray('0101')
+        with self.assertRaises(bitstring.CreationError):
+            _ = Bits(bitarray=ba, length=5)
+        with self.assertRaises(bitstring.CreationError):
+            _ = Bits(bitarray=ba, offset=5)
+        with self.assertRaises(bitstring.CreationError):
+            _ = Bits(ba, length=-1)
+
+    def testCreationFromMemoryview(self):
+        x = bytes(bytearray(range(20)))
+        m = memoryview(x[10:15])
+        b = Bits(m)
+        self.assertEqual(b.unpack('5*u8'), [10, 11, 12, 13, 14])
 
 
 class Initialisation(unittest.TestCase):
@@ -210,9 +241,9 @@ class Initialisation(unittest.TestCase):
 
 class Cut(unittest.TestCase):
     def testCut(self):
-        s = Bits(30)
-        for t in s.cut(3):
-            self.assertEqual(t, [0] * 3)
+        s = Bits('0b000111'*10)
+        for t in s.cut(6):
+            self.assertEqual(t.bin, '000111')
 
 
 class InterleavedExpGolomb(unittest.TestCase):
@@ -261,15 +292,15 @@ class FileBased(unittest.TestCase):
         self.d = Bits(filename=filename, offset=20, length=4)
 
     def testCreationWithOffset(self):
-        self.assertEqual(self.a, '0x0123456789abcdef')
-        self.assertEqual(self.b, '0x456789abcdef')
-        self.assertEqual(self.c, '0x5678')
+        self.assertEqual(str(self.a), '0x0123456789abcdef')
+        self.assertEqual(str(self.b), '0x456789abcdef')
+        self.assertEqual(str(self.c), '0x5678')
 
     def testBitOperators(self):
         x = self.b[4:20]
         self.assertEqual(x, '0x5678')
         self.assertEqual((x & self.c).hex, self.c.hex)
-        self.assertEqual(self.c ^ self.b[4:20], 16)
+        self.assertEqual(self.c ^ self.b[4:20], Bits(16))
         self.assertEqual(self.a[23:36] | self.c[3:], self.c[3:])
 
     def testAddition(self):
@@ -281,41 +312,6 @@ class FileBased(unittest.TestCase):
         x = BitArray(x)
         del x[12:24]
         self.assertEqual(x, '0x456abcdef587')
-        
-
-class Mmap(unittest.TestCase):
-    def setUp(self):
-        self.f = open(os.path.join(THIS_DIR, 'smalltestfile'), 'rb')
-
-    def tearDown(self):
-        self.f.close()
-
-    def testByteArrayEquivalence(self):
-        a = MmapByteArray(self.f)
-        self.assertEqual(a.bytelength, 8)
-        self.assertEqual(len(a), 8)
-        self.assertEqual(a[0], 0x01)
-        self.assertEqual(a[1], 0x23)
-        self.assertEqual(a[7], 0xef)
-        self.assertEqual(a[0:1], bytearray([1]))
-        self.assertEqual(a[:], bytearray([0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef]))
-        self.assertEqual(a[2:4], bytearray([0x45, 0x67]))
-
-    def testWithLength(self):
-        a = MmapByteArray(self.f, 3)
-        self.assertEqual(a[0], 0x01)
-        self.assertEqual(len(a), 3)
-
-    def testWithOffset(self):
-        a = MmapByteArray(self.f, None, 5)
-        self.assertEqual(len(a), 3)
-        self.assertEqual(a[0], 0xab)
-
-    def testWithLengthAndOffset(self):
-        a = MmapByteArray(self.f, 3, 3)
-        self.assertEqual(len(a), 3)
-        self.assertEqual(a[0], 0x67)
-        self.assertEqual(a[:], bytearray([0x67, 0x89, 0xab]))
 
 
 class Comparisons(unittest.TestCase):
@@ -351,36 +347,7 @@ class LongBoolConversion(unittest.TestCase):
     def testLongBool(self):
         a = Bits(1000)
         b = bool(a)
-        self.assertTrue(b is False)
-
-
-# Some basic tests for the private ByteStore classes
-
-class ByteStoreCreation(unittest.TestCase):
-
-    def testProperties(self):
-        a = ByteStore(bytearray(b'abc'))
-        self.assertEqual(a.bytelength, 3)
-        self.assertEqual(a.offset, 0)
-        self.assertEqual(a.bitlength, 24)
-        self.assertEqual(a.rawarray, b'abc')
-
-    def testGetBit(self):
-        a = ByteStore(bytearray([0x0f]))
-        self.assertEqual(a.getbit(0), False)
-        self.assertEqual(a.getbit(3), False)
-        self.assertEqual(a.getbit(4), True)
-        self.assertEqual(a.getbit(7), True)
-
-        b = ByteStore(bytearray([0x0f]), 7, 1)
-        self.assertEqual(b.getbit(2), False)
-        self.assertEqual(b.getbit(3), True)
-
-    def testGetByte(self):
-        a = ByteStore(bytearray(b'abcde'), 1, 13)
-        self.assertEqual(a.getbyte(0), 97)
-        self.assertEqual(a.getbyte(1), 98)
-        self.assertEqual(a.getbyte(4), 101)
+        self.assertTrue(b is True)
 
 
 class PadToken(unittest.TestCase):
@@ -412,7 +379,7 @@ class PadToken(unittest.TestCase):
 
     def testUnpackBug(self):
         t = Bits('0o755, ue=12, int3=-1')
-        a, b = t.unpack('pad:9, ue, int')
+        a, b = t.unpack('pad:9, ue, int3')
         self.assertEqual((a, b), (12, -1))
 
 
@@ -430,9 +397,9 @@ class ModifiedByAddingBug(unittest.TestCase):
         a = Bits(100)
         b = Bits(101)
         c = a + b
-        self.assertEqual(a, 100)
-        self.assertEqual(b, 101)
-        self.assertEqual(c, 201)
+        self.assertEqual(a, Bits(100))
+        self.assertEqual(b, Bits(101))
+        self.assertEqual(c, Bits(201))
 
 
 class WrongTypeBug(unittest.TestCase):
@@ -505,13 +472,13 @@ class ContainsBug(unittest.TestCase):
 
 class ByteStoreImmutablity(unittest.TestCase):
     
-    def testBitsDataStoreType(self):
-        a = Bits('0b1')
-        b = Bits('0b111')
-        c = a + b
-        self.assertEqual(type(a._datastore), ByteStore)
-        self.assertEqual(type(b._datastore), ByteStore)
-        self.assertEqual(type(c._datastore), ByteStore)
+    # def testBitsDataStoreType(self):
+    #     a = Bits('0b1')
+    #     b = Bits('0b111')
+    #     c = a + b
+    #     self.assertEqual(type(a._datastore), ByteStore)
+    #     self.assertEqual(type(b._datastore), ByteStore)
+    #     self.assertEqual(type(c._datastore), ByteStore)
 
     def testImmutabilityBugAppend(self):
         a = Bits('0b111')
@@ -521,7 +488,7 @@ class ByteStoreImmutablity(unittest.TestCase):
         self.assertEqual(c.bin, '101000')
         self.assertEqual(a.b3, '111')
         self.assertEqual(b.bin, '111000')
-        self.assertEqual(type(b._datastore), ByteStore)
+        # self.assertEqual(type(b._datastore), ByteStore)
 
     def testImmutabilityBugPrepend(self):
         a = Bits('0b111')
@@ -533,7 +500,7 @@ class ByteStoreImmutablity(unittest.TestCase):
 
     def testImmutabilityBugCreation(self):
         a = Bits()
-        self.assertEqual(type(a._datastore), ByteStore)
+        # self.assertEqual(type(a._datastore), ByteStore)
 
 
 class Lsb0Indexing(unittest.TestCase):
@@ -588,7 +555,7 @@ class Lsb0Indexing(unittest.TestCase):
     def testStartswith(self):
         a = Bits('0b0000000111')
         self.assertTrue(a.startswith('0b111'))
-        self.assertFalse(a.startswith(1))
+        self.assertFalse(a.startswith('0b0'))
         self.assertTrue(a.startswith('0b011', start=1))
         self.assertFalse(a.startswith('0b0111', end=3))
         self.assertTrue(a.startswith('0b0111', end=4))
@@ -674,7 +641,7 @@ class PrettyPrinting(unittest.TestCase):
         a = Bits('0b101011110000')
         s = io.StringIO()
         a.pp(stream=s)
-        self.assertEqual(s.getvalue(), ' 0: 10101111 0000\n')
+        self.assertEqual(s.getvalue(), ' 0: 10101111 0000   af 0\n')
 
         s = io.StringIO()
         a.pp('hex', stream=s)
@@ -687,7 +654,7 @@ class PrettyPrinting(unittest.TestCase):
     def testSmallWidth(self):
         a = Bits(20)
         s = io.StringIO()
-        a.pp(stream=s, width=5)
+        a.pp(fmt='b', stream=s, width=5)
         self.assertEqual(s.getvalue(), ' 0: 00000000\n'
                                        ' 8: 00000000\n'
                                        '16: 0000    \n')
@@ -701,7 +668,7 @@ class PrettyPrinting(unittest.TestCase):
     def testMultiLine(self):
         a = Bits(100)
         s = io.StringIO()
-        a.pp(sep=None, stream=s, width=80)
+        a.pp('bin', sep='', stream=s, width=80)
         self.assertEqual(s.getvalue(), '  0: 000000000000000000000000000000000000000000000000000000000000000000000000\n'
                                        ' 72: 0000000000000000000000000000                                            \n')
 
@@ -733,6 +700,14 @@ class PrettyPrinting(unittest.TestCase):
                                        ' 80: 11111111 11111111   ff ff\n'
                                        ' 96: 11111111 11111111   ff ff\n')
 
+        a = bytearray(range(0, 256))
+        b = Bits(bytes=a)
+        s = io.StringIO()
+        b.pp(stream=s, fmt='bytes')
+        self.assertEqual(s.getvalue(), """   0: ĀāĂă ĄąĆć ĈĉĊċ ČčĎď ĐđĒē ĔĕĖė ĘęĚě ĜĝĞğ  !"# $%&' ()*+ ,-./ 0123 4567 89:; <=>? @ABC DEFG HIJK LMNO PQRS TUVW XYZ[
+ 736: \]^_ `abc defg hijk lmno pqrs tuvw xyz{ |}~ſ ƀƁƂƃ ƄƅƆƇ ƈƉƊƋ ƌƍƎƏ ƐƑƒƓ ƔƕƖƗ Ƙƙƚƛ ƜƝƞƟ ƠơƢƣ ƤƥƦƧ ƨƩƪƫ ƬƭƮƯ ưƱƲƳ ƴƵƶƷ
+1472: Ƹƹƺƻ Ƽƽƾƿ ǀǁǂǃ ǄǅǆǇ ǈǉǊǋ ǌǍǎǏ ǐǑǒǓ ǔǕǖǗ ǘǙǚǛ ǜǝǞǟ ǠǡǢǣ ǤǥǦǧ ǨǩǪǫ ǬǭǮǯ ǰǱǲǳ ǴǵǶǷ ǸǹǺǻ ǼǽǾÿ                         \n""")
+
     def testGroupSizeErrors(self):
         a = Bits(120)
         with self.assertRaises(ValueError):
@@ -816,7 +791,7 @@ class PrettyPrinting_LSB0(unittest.TestCase):
     def test_bin(self):
         a = Bits(bin='1111 0000 0000 1111 1010')
         s = io.StringIO()
-        a.pp(stream=s, width=5)
+        a.pp('bin', stream=s, width=5)
         self.assertEqual(s.getvalue(), '11111010 : 0\n'
                                        '00000000 : 8\n'
                                        '    1111 :16\n')
@@ -827,3 +802,35 @@ class Copy(unittest.TestCase):
         s = Bits('0xc00dee')
         t = s.copy()
         self.assertEqual(s, t)
+
+
+class NativeEndianIntegers(unittest.TestCase):
+
+    def testUintne(self):
+        s = Bits(uintne=454, length=160)
+        t = Bits('uintne160=454')
+        self.assertEqual(s, t)
+
+    def testIntne(self):
+        s = Bits(intne=-1000, length=64)
+        t = Bits('intne:64=-1000')
+        self.assertEqual(s, t)
+
+
+class NonNativeEndianIntegers(unittest.TestCase):
+
+    def setUp(self) -> None:
+        bitstring.bits.byteorder = 'little' if bitstring.bits.byteorder == 'big' else 'little'
+
+    def tearDown(self) -> None:
+        self.setUp()
+
+    def testUintne(self):
+        s = Bits(uintne=454, length=160)
+        t = Bits('uintne160=454')
+        self.assertEqual(s, t)
+
+    def testIntne(self):
+        s = Bits(intne=-1000, length=64)
+        t = Bits('intne:64=-1000')
+        self.assertEqual(s, t)
